@@ -3,7 +3,7 @@ from __future__ import annotations
 from tokenize import group
 
 from matplotlib import cm
-from yaml import scan
+# from yaml import scan
 # from sympy import N
 from plankton.graph import SpatialGraph
 from plankton.pixelmaps import PixelMap, PixelMask
